# Fruit terms

Sketching out preferred display using https://mermaid.live - to propose changes, you are welcome to create MRs to this file, or to use https://mermaid.live or [Drupal's Gitlab Snippet functionality](https://git.drupalcode.org/dashboard/snippets). Or just attach a photo of a drawing to #3259669! :)

Here's an example migration with some needlessly convoluted process including pseudofields:

```yaml
langcode: en
status: true
dependencies: {  }
id: fruit_terms
class: null
field_plugin_method: null
cck_plugin_method: null
migration_tags: {  }
migration_group: default
label: 'Fruit Terms'
source:
  plugin: embedded_data
  data_rows:
    -
      name: Apple
      weight: 85g
    -
      name: Banana
      weight: 95g
    -
      name: Orange
      weight: 100g
  ids:
    name:
      type: string
  constants:
    vocabulary: fruit
process:
  name: name
  vid: constants/vocabulary
  field_weight: weight
  _first_letter:
    -
      plugin: str_replace
      regex: true
      source: name
      search: '/^(.).*/'
      replace: '$1'
  _ordinal:
    -
      plugin: callback
      source: '@_first_letter'
      callable: ord
  field_section: '@_ordinal'
destination:
  plugin: 'entity:taxonomy_term'
migration_dependencies:
  required: {  }
  optional: {  }
```

Here's what is currently output as of [b56dd95cc](https://git.drupalcode.org/sandbox/grobot-3257623/-/blob/b56dd95cc8bb7727bfad78c44b6f08cad7724247).

```mermaid
# 👀 drupal/migrate_visualize
flowchart LR

subgraph Source[fa:fa-cloud-download Source]
  src_name["name"]
  src_constants_vocabulary["constants/vocabulary"]
  src_weight["weight"]
  src___first_letter["@_first_letter"]
  src___ordinal["@_ordinal"]
end

subgraph Destination[fa:fa-cloud-upload Destination]
  dst_name["name"]
  dst_vid["vid"]
  dst_field_weight["field_weight"]
  dst__first_letter["_first_letter"]
  dst__ordinal["_ordinal"]
  dst_field_section["field_section"]
end

subgraph Combine[fa:fa-cloud-upload Combine]
  proc_src_b068931cc450442b63f5b3d276ea4297["source: name"]
  proc_src_f405d7ab4b93471870f6dd89f79f2bda["source: constants/vocabulary"]
  proc_src_7edabf994b76a00cbc60c95af337db8f["source: weight"]
  proc_src_7920f0b827aa8c686b9ecc50efb88058["source: @_first_letter"]
  proc_src_f970863a02110f3d0262132ad70aa702["source: @_ordinal"]
end

subgraph Process[fa:fa-cloud-upload Process]
  proc_plugs_b068931cc450442b63f5b3d276ea4297["this,that"]
  proc_plugs_f405d7ab4b93471870f6dd89f79f2bda["this,that"]
  proc_plugs_7edabf994b76a00cbc60c95af337db8f["this,that"]
  proc_plugs_7920f0b827aa8c686b9ecc50efb88058["this,that"]
  proc_plugs_f970863a02110f3d0262132ad70aa702["this,that"]
end

src_name -.-> proc_src_b068931cc450442b63f5b3d276ea4297
proc_src_b068931cc450442b63f5b3d276ea4297 -.-> proc_plugs_b068931cc450442b63f5b3d276ea4297
proc_plugs_b068931cc450442b63f5b3d276ea4297 -.-> dst__first_letter
src_constants_vocabulary -.-> proc_src_f405d7ab4b93471870f6dd89f79f2bda
proc_src_f405d7ab4b93471870f6dd89f79f2bda -.-> proc_plugs_f405d7ab4b93471870f6dd89f79f2bda
proc_plugs_f405d7ab4b93471870f6dd89f79f2bda -.-> dst_vid
src_weight -.-> proc_src_7edabf994b76a00cbc60c95af337db8f
proc_src_7edabf994b76a00cbc60c95af337db8f -.-> proc_plugs_7edabf994b76a00cbc60c95af337db8f
proc_plugs_7edabf994b76a00cbc60c95af337db8f -.-> dst_field_weight
src___first_letter -.-> proc_src_7920f0b827aa8c686b9ecc50efb88058
proc_src_7920f0b827aa8c686b9ecc50efb88058 -.-> proc_plugs_7920f0b827aa8c686b9ecc50efb88058
proc_plugs_7920f0b827aa8c686b9ecc50efb88058 -.-> dst__ordinal
src___ordinal -.-> proc_src_f970863a02110f3d0262132ad70aa702
proc_src_f970863a02110f3d0262132ad70aa702 -.-> proc_plugs_f970863a02110f3d0262132ad70aa702
proc_plugs_f970863a02110f3d0262132ad70aa702 -.-> dst_field_section
```

Drupal.org's Gitlab shows the above as a Mermaid syntax error, but it looks [like this](fruit-terms-b56dd95cc.png) when using the module.

----

## Sketching intended output

Charts below here are generated by hand, sketching out how it *might* be useful.

Here's a version where the processes are displayed as steps.

```mermaid
flowchart LR

subgraph Source[fa:fa-cloud-download Source]
  src_name["name"]
  src_weight["weight"]
  src_constants_vocabulary["constants/vocabulary"]
end

subgraph Destination[fa:fa-cloud-upload Destination]
  dst_name["name"]
  dst_vid["vid"]
  dst_field_weight["field_weight"]
  dst__first_letter["_first_letter"]
  dst__ordinal["_ordinal"]
  dst_field_section["field_section"]
end

subgraph Process[fa:fa-cloud-upload Process]
  proc_src_name["get: name"]
  proc_src_weight["get: weight"]
  proc_src_section["get: '@_ordinal'"]
  src___first_letter["str_replace"]
  src___ordinal["callable"]
  proc_src_vocab["get: constants/vocabulary"]
end

src_name -.-> proc_src_name
proc_src_name -.-> dst_name
src_weight -.-> proc_src_weight
proc_src_weight -.-> dst_field_weight
proc_src_name -.-> src___first_letter
src___first_letter -.-> dst__first_letter
src___first_letter -.-> src___ordinal
src___ordinal -.-> dst__ordinal
src___ordinal -.-> proc_src_section
proc_src_section -.-> dst_field_section
src_constants_vocabulary -.-> proc_src_vocab
proc_src_vocab -.-> dst_vid
```

Here's a version which is also smart enough to discard pseudofields at destination (by implementing some magic around having seen a pseudofield get used in another destination, or looking for `_` prefix perhaps?).

```mermaid
flowchart LR

subgraph Source[fa:fa-cloud-download Source]
  src_name["name"]
  src_weight["weight"]
  src_constants_vocabulary["constants/vocabulary"]
end

subgraph Destination[fa:fa-cloud-upload Destination]
  dst_name["name"]
  dst_vid["vid"]
  dst_field_weight["field_weight"]
  dst_field_section["field_section"]
end

subgraph Process[fa:fa-cloud-upload Process]
  proc_src_name["get: name"]
  proc_src_weight["get: weight"]
  proc_src_section["get: '@_ordinal'"]
  src___first_letter["str_replace"]
  src___ordinal["callable"]
  proc_src_vocab["get: constants/vocabulary"]
end

src_name -.-> proc_src_name
proc_src_name -.-> dst_name
src_weight -.-> proc_src_weight
proc_src_weight -.-> dst_field_weight
proc_src_name -.-> src___first_letter
src___first_letter -.-> src___ordinal
src___ordinal -.-> proc_src_section
proc_src_section -.-> dst_field_section
src_constants_vocabulary -.-> proc_src_vocab
proc_src_vocab -.-> dst_vid
```
